import unittest
from unittest.mock import MagicMock

from sensory_cloud.config import Config, CloudHost
from sensory_cloud.generated.v1.management.enrollment_pb2 import (
    CreateEnrollmentGroupRequest,
    EnrollmentGroupResponse,
    EnrollmentResponse,
    GetEnrollmentGroupsResponse,
    GetEnrollmentsResponse,
)
from sensory_cloud.services.management_service import ManagementService
from sensory_cloud.generated.v1.management.enrollment_pb2_grpc import (
    EnrollmentServiceStub,
)
from sensory_cloud.token_manager import ITokenManager, Metadata


class MockTokenManager(ITokenManager):
    def __init__(self, token: str = "token"):
        self._token = token

    def get_token(self) -> str:
        return self._token

    def get_authorization_metadata(self) -> Metadata:
        token: str = self.get_token()
        return (("authorization", f"Bearer {token}"),)


class MockManagementService(ManagementService):
    def __init__(
        self,
        config: Config,
        token_manager: ITokenManager,
        enrollment_client: EnrollmentServiceStub,
    ):

        self._config = config
        self._token_manager = token_manager
        self._enrollment_client = enrollment_client


class ManagmentServiceTest(unittest.TestCase):

    cloud_host: CloudHost = CloudHost(host="domain.name")
    config: Config = Config(cloud_host=cloud_host, tenant_id="tenant-id")
    config.connect()

    enrollment_client: EnrollmentServiceStub = EnrollmentServiceStub(
        channel=config.channel
    )

    token_manager: MockTokenManager = MockTokenManager()

    def test_get_enrollments(self):
        self.config.connect()

        response: GetEnrollmentsResponse = GetEnrollmentsResponse(
            enrollments=[
                EnrollmentResponse(
                    description="enrollment", didEnrollWithLiveness=False
                )
            ]
        )

        self.enrollment_client.GetEnrollments = MagicMock(return_value=response)

        management_service: MockManagementService = MockManagementService(
            config=self.config,
            token_manager=self.token_manager,
            enrollment_client=self.enrollment_client,
        )

        enrollment_response: GetEnrollmentsResponse = (
            management_service.get_enrollments(user_id="user-id")
        )

        self.assertEqual(response, enrollment_response)

        self.config.channel.close()

    def test_get_enrollment_groups(self):
        self.config.connect()

        user_id: str = "user-id"

        response: GetEnrollmentGroupsResponse = GetEnrollmentGroupsResponse(
            enrollmentGroups=[
                EnrollmentGroupResponse(
                    description="enrollment group", modelName="my-model"
                )
            ]
        )

        self.enrollment_client.GetEnrollmentGroups = MagicMock(return_value=response)

        management_service: MockManagementService = MockManagementService(
            config=self.config,
            token_manager=self.token_manager,
            enrollment_client=self.enrollment_client,
        )

        enrollment_group_response: GetEnrollmentGroupsResponse = (
            management_service.get_enrollment_groups(user_id="user-id")
        )

        self.assertEqual(response, enrollment_group_response)

        self.config.channel.close()

    def test_create_enrollment_group(self):
        self.config.connect()

        response: EnrollmentGroupResponse = EnrollmentGroupResponse(
            description="enrollment group", modelName="my-model"
        )

        self.enrollment_client.CreateEnrollmentGroup = MagicMock(return_value=response)

        management_service: MockManagementService = MockManagementService(
            config=self.config,
            token_manager=self.token_manager,
            enrollment_client=self.enrollment_client,
        )

        enrollment_group_response: EnrollmentGroupResponse = (
            management_service.create_enrollment_group(
                user_id="user-id",
                group_id="group-id",
                group_name="group-name",
                description="enrollment group",
                model_name="model-name",
                enrollment_ids=["foo", "bar", "baz"],
            )
        )

        self.assertEqual(response, enrollment_group_response)

        self.config.channel.close()

    def test_append_enrollment_group(self):
        self.config.connect()

        response: EnrollmentGroupResponse = EnrollmentGroupResponse(
            description="enrollment group", modelName="my-model"
        )

        self.enrollment_client.AppendEnrollmentGroup = MagicMock(return_value=response)

        management_service: MockManagementService = MockManagementService(
            config=self.config,
            token_manager=self.token_manager,
            enrollment_client=self.enrollment_client,
        )

        enrollment_group_response: EnrollmentGroupResponse = (
            management_service.append_enrollment_group(
                group_id="group-id", enrollment_ids=["foo", "bar", "baz"]
            )
        )

        self.assertEqual(response, enrollment_group_response)

        self.config.channel.close()

    def test_delete_enrollment(self):
        self.config.connect()

        response: EnrollmentResponse = EnrollmentResponse(
            description="enrollment", modelName="my-model"
        )

        self.enrollment_client.DeleteEnrollment = MagicMock(return_value=response)

        management_service: MockManagementService = MockManagementService(
            config=self.config,
            token_manager=self.token_manager,
            enrollment_client=self.enrollment_client,
        )

        delete_enrollment_response: EnrollmentResponse = (
            management_service.delete_enrollment(id="id")
        )

        self.assertEqual(response, delete_enrollment_response)

        self.config.channel.close()

    def test_delete_enrollment_group(self):
        self.config.connect()

        response: EnrollmentGroupResponse = EnrollmentGroupResponse(
            description="enrollment", modelName="my-model"
        )

        self.enrollment_client.DeleteEnrollmentGroup = MagicMock(return_value=response)

        management_service: MockManagementService = MockManagementService(
            config=self.config,
            token_manager=self.token_manager,
            enrollment_client=self.enrollment_client,
        )

        delete_enrollment_group_response: EnrollmentGroupResponse = (
            management_service.delete_enrollment_group(group_id="group-id")
        )

        self.assertEqual(response, delete_enrollment_group_response)

        self.config.channel.close()


if __name__ == "__main__":
    unittest.main()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from sensory_cloud.generated.v1.management import enrollment_pb2 as v1_dot_management_dot_enrollment__pb2


class EnrollmentServiceStub(object):
    """Service to manage Enrollments in the database
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetEnrollments = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/GetEnrollments',
                request_serializer=v1_dot_management_dot_enrollment__pb2.GetEnrollmentsRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.GetEnrollmentsResponse.FromString,
                )
        self.GetEnrollmentGroups = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/GetEnrollmentGroups',
                request_serializer=v1_dot_management_dot_enrollment__pb2.GetEnrollmentsRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.GetEnrollmentGroupsResponse.FromString,
                )
        self.CreateEnrollmentGroup = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/CreateEnrollmentGroup',
                request_serializer=v1_dot_management_dot_enrollment__pb2.CreateEnrollmentGroupRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
                )
        self.AppendEnrollmentGroup = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/AppendEnrollmentGroup',
                request_serializer=v1_dot_management_dot_enrollment__pb2.AppendEnrollmentGroupRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
                )
        self.DeleteEnrollment = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/DeleteEnrollment',
                request_serializer=v1_dot_management_dot_enrollment__pb2.DeleteEnrollmentRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.EnrollmentResponse.FromString,
                )
        self.DeleteEnrollmentGroup = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/DeleteEnrollmentGroup',
                request_serializer=v1_dot_management_dot_enrollment__pb2.DeleteEnrollmentGroupRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
                )
        self.UpdateEnrollment = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/UpdateEnrollment',
                request_serializer=v1_dot_management_dot_enrollment__pb2.UpdateEnrollmentRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.EnrollmentResponse.FromString,
                )
        self.UpdateEnrollmentGroup = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/UpdateEnrollmentGroup',
                request_serializer=v1_dot_management_dot_enrollment__pb2.UpdateEnrollmentGroupRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
                )
        self.RemoveEnrollmentsFromGroup = channel.unary_unary(
                '/sensory.api.v1.management.EnrollmentService/RemoveEnrollmentsFromGroup',
                request_serializer=v1_dot_management_dot_enrollment__pb2.RemoveEnrollmentsRequest.SerializeToString,
                response_deserializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
                )


class EnrollmentServiceServicer(object):
    """Service to manage Enrollments in the database
    """

    def GetEnrollments(self, request, context):
        """Get enrollments from the database that match the specified criteria
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetEnrollmentGroups(self, request, context):
        """Get all enrollment groups that match the specified criteria
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateEnrollmentGroup(self, request, context):
        """Creates a new enrollment group without any associated enrollments
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AppendEnrollmentGroup(self, request, context):
        """Appends an enrollment to an enrollment group
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteEnrollment(self, request, context):
        """Deletes an enrollment from the database
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        A user may not delete their last enrollment, as this would lead to a situation where a user could
        lock themselves out of the system forever.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteEnrollmentGroup(self, request, context):
        """Deletes an enrollment group from the database
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateEnrollment(self, request, context):
        """Updates the name of an enrollment
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateEnrollmentGroup(self, request, context):
        """Updates the name of an enrollment group
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveEnrollmentsFromGroup(self, request, context):
        """Removes a list of enrollments from an enrollment group
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EnrollmentServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetEnrollments': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEnrollments,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.GetEnrollmentsRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.GetEnrollmentsResponse.SerializeToString,
            ),
            'GetEnrollmentGroups': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEnrollmentGroups,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.GetEnrollmentsRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.GetEnrollmentGroupsResponse.SerializeToString,
            ),
            'CreateEnrollmentGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateEnrollmentGroup,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.CreateEnrollmentGroupRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.SerializeToString,
            ),
            'AppendEnrollmentGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.AppendEnrollmentGroup,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.AppendEnrollmentGroupRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.SerializeToString,
            ),
            'DeleteEnrollment': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteEnrollment,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.DeleteEnrollmentRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.EnrollmentResponse.SerializeToString,
            ),
            'DeleteEnrollmentGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteEnrollmentGroup,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.DeleteEnrollmentGroupRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.SerializeToString,
            ),
            'UpdateEnrollment': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateEnrollment,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.UpdateEnrollmentRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.EnrollmentResponse.SerializeToString,
            ),
            'UpdateEnrollmentGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateEnrollmentGroup,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.UpdateEnrollmentGroupRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.SerializeToString,
            ),
            'RemoveEnrollmentsFromGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveEnrollmentsFromGroup,
                    request_deserializer=v1_dot_management_dot_enrollment__pb2.RemoveEnrollmentsRequest.FromString,
                    response_serializer=v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'sensory.api.v1.management.EnrollmentService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class EnrollmentService(object):
    """Service to manage Enrollments in the database
    """

    @staticmethod
    def GetEnrollments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/GetEnrollments',
            v1_dot_management_dot_enrollment__pb2.GetEnrollmentsRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.GetEnrollmentsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetEnrollmentGroups(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/GetEnrollmentGroups',
            v1_dot_management_dot_enrollment__pb2.GetEnrollmentsRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.GetEnrollmentGroupsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateEnrollmentGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/CreateEnrollmentGroup',
            v1_dot_management_dot_enrollment__pb2.CreateEnrollmentGroupRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AppendEnrollmentGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/AppendEnrollmentGroup',
            v1_dot_management_dot_enrollment__pb2.AppendEnrollmentGroupRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteEnrollment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/DeleteEnrollment',
            v1_dot_management_dot_enrollment__pb2.DeleteEnrollmentRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.EnrollmentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteEnrollmentGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/DeleteEnrollmentGroup',
            v1_dot_management_dot_enrollment__pb2.DeleteEnrollmentGroupRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateEnrollment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/UpdateEnrollment',
            v1_dot_management_dot_enrollment__pb2.UpdateEnrollmentRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.EnrollmentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateEnrollmentGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/UpdateEnrollmentGroup',
            v1_dot_management_dot_enrollment__pb2.UpdateEnrollmentGroupRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveEnrollmentsFromGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.management.EnrollmentService/RemoveEnrollmentsFromGroup',
            v1_dot_management_dot_enrollment__pb2.RemoveEnrollmentsRequest.SerializeToString,
            v1_dot_management_dot_enrollment__pb2.EnrollmentGroupResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import typing
from enum import Enum

from sensory_cloud.config import Config
from sensory_cloud.token_manager import ITokenManager, Metadata

import sensory_cloud.generated.v1.video.video_pb2_grpc as video_pb2_grpc
import sensory_cloud.generated.v1.video.video_pb2 as video_pb2


class VideoRequest(Enum):
    video_pb2.CreateEnrollmentRequest
    video_pb2.AuthenticateRequest
    video_pb2.ValidateRecognitionRequest


class RequestConfig(Enum):
    video_pb2.CreateEnrollmentConfig
    video_pb2.AuthenticateConfig
    video_pb2.ValidateRecognitionConfig


class RequestIterator:
    """
    The RequestIterator class facilitates the request streams that are sent to the
    grpc server.  There are three possible video request types and request configurations
    that are given by the VideoRequest and RequestConfig enums respectively.  The first
    request sent must be a configuration request and all subsequent requests contain the image
    content being streamed.
    """

    first_request: bool = True

    def __init__(
        self,
        video_request: VideoRequest,
        request_config: RequestConfig,
        video_stream_iterator: typing.Iterable[bytes],
    ):
        """
        Constructor method for the RequestIterator class

        Arguments:
            video_request: VideoRequest enum denoting which type of request is being sent
            request_config: RequestConfig enum containing the initial request configuration
            video_stream_iterator: Iterator containing image bytes
        """

        self._video_request = video_request
        self._request_config = request_config
        self._video_stream_iterator = video_stream_iterator

    def __iter__(self):
        return self

    def __next__(self):
        if self.first_request:
            self.first_request = False
            return self._video_request(config=self._request_config)
        else:
            image_content = next(self._video_stream_iterator)
            return self._video_request(imageContent=image_content)


class VideoService:
    """
    Class that handles all video requests to Sensory Cloud.
    """

    def __init__(self, config: Config, token_manager: ITokenManager):
        """
        Constructor method for the VideoService class

        Arguments:
            config: Config object containing the relevant grpc connection information
            token_manager: ITokenManager object that generates and returns JWT metadata
        """

        self._config: Config = config
        self._token_manager: ITokenManager = token_manager
        self._video_models_client: video_pb2_grpc.VideoModelsStub = (
            video_pb2_grpc.VideoModelsStub(config.channel)
        )
        self._video_biometrics_client: video_pb2_grpc.VideoBiometricsStub = (
            video_pb2_grpc.VideoBiometricsStub(config.channel)
        )
        self._video_recognition_client: video_pb2_grpc.VideoRecognitionStub = (
            video_pb2_grpc.VideoRecognitionStub(config.channel)
        )

    def get_models(self) -> video_pb2.GetModelsResponse:
        """
        Method that fetches all the video models supported by your instance of Sensory Cloud.

        Returns:
            A GetModelsResponse containing information about all video models
        """

        metadata: Metadata = self._token_manager.get_authorization_metadata()
        request: video_pb2.GetModelsRequest = video_pb2.GetModelsRequest()

        response: video_pb2.GetModelsResponse = self._video_models_client.GetModels(
            request=request, metadata=metadata
        )

        return response

    def stream_enrollment(
        self,
        description: str,
        user_id: str,
        model_name: str,
        device_id: str,
        is_liveness_enabled: bool,
        video_stream_iterator: typing.Iterable[bytes],
        threshold: video_pb2.RecognitionThreshold = video_pb2.RecognitionThreshold.Value(
            "HIGH"
        ),
        num_liveness_frames_required: int = 0,
    ) -> typing.Iterable[video_pb2.CreateEnrollmentResponse]:
        """
        Stream video to Sensory Cloud as a means for user enrollment.
        Only biometric-typed models are supported by the method.

        Arguments:
            description: String containing a description of the enrollment
            user_id: String containing the user id
            model_name: String containing the model name
            device_id: String containing the device id
            is_liveness_enabled: Boolean indicating whether or not liveness is enabled
            video_stream_iterator: Iterator of image bytes
            threshold: The liveness threshold (if liveness is enabled)
                default = RecognitionThreshold = RecognitionThreshold.Value("HIGH")
            num_liveness_frames_required: Integer indicating the number of live frames are required for a successful enrollment
                default = 0, which requires all frames to be live

        Returns:
            An iterator of CreateEnrollmentResponse objects
        """

        config: video_pb2.CreateEnrollmentConfig = video_pb2.CreateEnrollmentConfig(
            description=description,
            userId=user_id,
            deviceId=device_id,
            modelName=model_name,
            isLivenessEnabled=is_liveness_enabled,
            livenessThreshold=threshold,
            numLivenessFramesRequired=num_liveness_frames_required,
        )

        request_iterator: RequestIterator = RequestIterator(
            video_request=video_pb2.CreateEnrollmentRequest,
            request_config=config,
            video_stream_iterator=video_stream_iterator,
        )

        metadata: Metadata = self._token_manager.get_authorization_metadata()

        enrollment_stream: typing.Iterable[
            video_pb2.CreateEnrollmentResponse
        ] = self._video_biometrics_client.CreateEnrollment(
            request_iterator=request_iterator, metadata=metadata
        )

        return enrollment_stream

    def stream_authentication(
        self,
        enrollment_id: str,
        is_liveness_enabled: bool,
        video_stream_iterator: typing.Iterable[bytes],
        threshold: video_pb2.RecognitionThreshold = video_pb2.RecognitionThreshold.Value(
            "HIGH"
        ),
    ) -> typing.Iterable[video_pb2.AuthenticateResponse]:
        """
        Authenticate against an existing video enrollment in Sensory Cloud.
        Only biometric-typed models are supported by the method.

        Arguments:
            enrollment_id: String containing the enrollment id to authenticate on
            is_liveness_enabled: Boolean indicating whether or not liveness is enabled
            video_stream_iterator: Iterator of audio bytes
            threshold: The liveness threshold (if liveness is enabled)
                default = RecognitionThreshold = RecognitionThreshold.Value("HIGH")

        Returns:
            An iterator of AuthenticateResponse objects
        """

        config: video_pb2.AuthenticateConfig = video_pb2.AuthenticateConfig(
            enrollmentId=enrollment_id,
            isLivenessEnabled=is_liveness_enabled,
            livenessThreshold=threshold,
        )

        request_iterator: RequestIterator = RequestIterator(
            video_request=video_pb2.AuthenticateRequest,
            request_config=config,
            video_stream_iterator=video_stream_iterator,
        )

        metadata: Metadata = self._token_manager.get_authorization_metadata()

        authenticate_stream: typing.Iterable[
            video_pb2.AuthenticateResponse
        ] = self._video_biometrics_client.Authenticate(
            request_iterator=request_iterator, metadata=metadata
        )

        return authenticate_stream

    def stream_group_authentication(
        self,
        enrollment_group_id: str,
        is_liveness_enabled: bool,
        video_stream_iterator: typing.Iterable[bytes],
        threshold: video_pb2.RecognitionThreshold = video_pb2.RecognitionThreshold.Value(
            "HIGH"
        ),
    ) -> typing.Iterable[video_pb2.AuthenticateResponse]:
        """
        Authenticate against an existing video enrollment group in Sensory Cloud.
        Only biometric-typed models are supported by the method.

        Arguments:
            enrollment_group_id: String containing the enrollment group id to authenticate on
            is_liveness_enabled: Boolean indicating whether or not liveness is enabled
            video_stream_iterator: Iterator of audio bytes
            threshold: The liveness threshold (if liveness is enabled)
                default = RecognitionThreshold = RecognitionThreshold.Value("HIGH")

        Returns:
            An iterator of AuthenticateResponse objects
        """

        config: video_pb2.AuthenticateConfig = video_pb2.AuthenticateConfig(
            enrollmentGroupId=enrollment_group_id,
            isLivenessEnabled=is_liveness_enabled,
            livenessThreshold=threshold,
        )

        request_iterator: RequestIterator = RequestIterator(
            video_request=video_pb2.AuthenticateRequest,
            request_config=config,
            video_stream_iterator=video_stream_iterator,
        )

        metadata: Metadata = self._token_manager.get_authorization_metadata()

        authenticate_stream: typing.Iterable[
            video_pb2.AuthenticateResponse
        ] = self._video_biometrics_client.Authenticate(
            request_iterator=request_iterator, metadata=metadata
        )

        return authenticate_stream

    def stream_liveness_recognition(
        self,
        user_id: str,
        model_name: str,
        video_stream_iterator: typing.Iterable[bytes],
        threshold: video_pb2.RecognitionThreshold = video_pb2.RecognitionThreshold.Value(
            "HIGH"
        ),
    ) -> typing.Iterable[video_pb2.LivenessRecognitionResponse]:
        """
        Method that streams images to Sensory Cloud in order to recognize "liveness" of a particular image

        Arguments:
            user_id: String containing user id
            model_name: String containing the model name
            video_stream_iterator: Iterator of audio bytes
            threshold: The liveness threshold (if liveness is enabled)
                default = RecognitionThreshold = RecognitionThreshold.Value("HIGH")

        Returns:
            An iterator containing LivenessRecognitionResponse objects
        """

        config: video_pb2.ValidateRecognitionConfig = (
            video_pb2.ValidateRecognitionConfig(
                userId=user_id, modelName=model_name, threshold=threshold
            )
        )

        request_iterator: RequestIterator = RequestIterator(
            video_request=video_pb2.ValidateRecognitionRequest,
            request_config=config,
            video_stream_iterator=video_stream_iterator,
        )

        metadata: Metadata = self._token_manager.get_authorization_metadata()

        recognition_stream: typing.Iterable[
            video_pb2.LivenessRecognitionResponse
        ] = self._video_recognition_client.ValidateLiveness(
            request_iterator=request_iterator, metadata=metadata
        )

        return recognition_stream



from setuptools import find_packages, setup


with open("requirements.txt", "r") as requirements_txt:
    requirements = requirements_txt.read().splitlines()

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="sensory_cloud",
    version="1.0.0",
    description="Python SDK for Sensory Cloud",
    author="Jonathan Hersch",
    author_email="jhersch@sensoryinc.com",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=requirements,
    python_requires='>=3.6',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Sensory-Cloud/python-sdk",
    license_files = ('LICENSE.txt',),
)


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/management/enrollment.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from sensory_cloud.generated.validate import validate_pb2 as validate_dot_validate__pb2
from sensory_cloud.generated.common import common_pb2 as common_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='v1/management/enrollment.proto',
  package='sensory.api.v1.management',
  syntax='proto3',
  serialized_options=b'\n!ai.sensorycloud.api.v1.managementB%SensoryApiV1ManagementEnrollmentProtoP\001Z?gitlab.com/sensory-cloud/server/titan.git/pkg/api/v1/management',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1ev1/management/enrollment.proto\x12\x19sensory.api.v1.management\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17validate/validate.proto\x1a\x13\x63ommon/common.proto\"2\n\x15GetEnrollmentsRequest\x12\x19\n\x06userId\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\"x\n\x16GetEnrollmentsResponse\x12\x42\n\x0b\x65nrollments\x18\x01 \x03(\x0b\x32-.sensory.api.v1.management.EnrollmentResponse\x12\x1a\n\x12isRequestorTrusted\x18\x02 \x01(\x08\"\xb9\x03\n\x12\x45nrollmentResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12-\n\tcreatedAt\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\tupdatedAt\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x11\n\tmodelName\x18\x05 \x01(\t\x12\x30\n\tmodelType\x18\x06 \x01(\x0e\x32\x1d.sensory.api.common.ModelType\x12\x14\n\x0cmodelVersion\x18\x07 \x01(\t\x12\x10\n\x08\x64\x65viceId\x18\x08 \x01(\t\x12\x0e\n\x06userId\x18\t \x01(\t\x12\x41\n\x0b\x63ompression\x18\n \x01(\x0b\x32,.sensory.api.common.CompressionConfiguration\x12\x12\n\ndeviceName\x18\x0b \x01(\t\x12\x1d\n\x15\x64idEnrollWithLiveness\x18\x0c \x01(\x08\x12\x13\n\x0breferenceId\x18\r \x01(\t\x12\x1c\n\x14\x64idEnrollAsBiometric\x18\x0e \x01(\x08\"k\n\x1bGetEnrollmentGroupsResponse\x12L\n\x10\x65nrollmentGroups\x18\x01 \x03(\x0b\x32\x32.sensory.api.v1.management.EnrollmentGroupResponse\"\xd5\x02\n\x17\x45nrollmentGroupResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12-\n\tcreatedAt\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\tupdatedAt\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x11\n\tmodelName\x18\x06 \x01(\t\x12\x30\n\tmodelType\x18\x07 \x01(\x0e\x32\x1d.sensory.api.common.ModelType\x12\x14\n\x0cmodelVersion\x18\x08 \x01(\t\x12\x0e\n\x06userId\x18\t \x01(\t\x12\x42\n\x0b\x65nrollments\x18\n \x03(\x0b\x32-.sensory.api.v1.management.EnrollmentResponse\"\xbf\x01\n\x1c\x43reateEnrollmentGroupRequest\x12\x15\n\x02id\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x18\n\x04name\x18\x02 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01\x18\xff\x01\x12\x1d\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x08\xfa\x42\x05r\x03\x18\xff\x07\x12\x1d\n\tmodelName\x18\x04 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01\x18\xff\x01\x12\x19\n\x06userId\x18\x05 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x15\n\renrollmentIds\x18\x06 \x03(\t\"Q\n\x1c\x41ppendEnrollmentGroupRequest\x12\x1a\n\x07groupId\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x15\n\renrollmentIds\x18\x02 \x03(\t\"/\n\x17\x44\x65leteEnrollmentRequest\x12\x14\n\x02id\x18\x01 \x01(\tB\x08\xfa\x42\x05r\x03\xb0\x01\x01\"5\n\x1c\x44\x65leteEnrollmentGroupRequest\x12\x15\n\x02id\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\"N\n\x17UpdateEnrollmentRequest\x12\x14\n\x02id\x18\x01 \x01(\tB\x08\xfa\x42\x05r\x03\xb0\x01\x01\x12\x1d\n\x0b\x64\x65scription\x18\x02 \x01(\tB\x08\xfa\x42\x05r\x03\x18\xff\x07\"N\n\x1cUpdateEnrollmentGroupRequest\x12\x15\n\x02id\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x17\n\x04name\x18\x02 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\"W\n\x18RemoveEnrollmentsRequest\x12\x1a\n\x07groupId\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x1f\n\renrollmentIds\x18\x02 \x03(\tB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x32\xb0\t\n\x11\x45nrollmentService\x12w\n\x0eGetEnrollments\x12\x30.sensory.api.v1.management.GetEnrollmentsRequest\x1a\x31.sensory.api.v1.management.GetEnrollmentsResponse\"\x00\x12\x81\x01\n\x13GetEnrollmentGroups\x12\x30.sensory.api.v1.management.GetEnrollmentsRequest\x1a\x36.sensory.api.v1.management.GetEnrollmentGroupsResponse\"\x00\x12\x86\x01\n\x15\x43reateEnrollmentGroup\x12\x37.sensory.api.v1.management.CreateEnrollmentGroupRequest\x1a\x32.sensory.api.v1.management.EnrollmentGroupResponse\"\x00\x12\x86\x01\n\x15\x41ppendEnrollmentGroup\x12\x37.sensory.api.v1.management.AppendEnrollmentGroupRequest\x1a\x32.sensory.api.v1.management.EnrollmentGroupResponse\"\x00\x12w\n\x10\x44\x65leteEnrollment\x12\x32.sensory.api.v1.management.DeleteEnrollmentRequest\x1a-.sensory.api.v1.management.EnrollmentResponse\"\x00\x12\x86\x01\n\x15\x44\x65leteEnrollmentGroup\x12\x37.sensory.api.v1.management.DeleteEnrollmentGroupRequest\x1a\x32.sensory.api.v1.management.EnrollmentGroupResponse\"\x00\x12w\n\x10UpdateEnrollment\x12\x32.sensory.api.v1.management.UpdateEnrollmentRequest\x1a-.sensory.api.v1.management.EnrollmentResponse\"\x00\x12\x86\x01\n\x15UpdateEnrollmentGroup\x12\x37.sensory.api.v1.management.UpdateEnrollmentGroupRequest\x1a\x32.sensory.api.v1.management.EnrollmentGroupResponse\"\x00\x12\x87\x01\n\x1aRemoveEnrollmentsFromGroup\x12\x33.sensory.api.v1.management.RemoveEnrollmentsRequest\x1a\x32.sensory.api.v1.management.EnrollmentGroupResponse\"\x00\x42\x8d\x01\n!ai.sensorycloud.api.v1.managementB%SensoryApiV1ManagementEnrollmentProtoP\x01Z?gitlab.com/sensory-cloud/server/titan.git/pkg/api/v1/managementb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,validate_dot_validate__pb2.DESCRIPTOR,common_dot_common__pb2.DESCRIPTOR,])




_GETENROLLMENTSREQUEST = _descriptor.Descriptor(
  name='GetEnrollmentsRequest',
  full_name='sensory.api.v1.management.GetEnrollmentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='userId', full_name='sensory.api.v1.management.GetEnrollmentsRequest.userId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=140,
  serialized_end=190,
)


_GETENROLLMENTSRESPONSE = _descriptor.Descriptor(
  name='GetEnrollmentsResponse',
  full_name='sensory.api.v1.management.GetEnrollmentsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='enrollments', full_name='sensory.api.v1.management.GetEnrollmentsResponse.enrollments', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='isRequestorTrusted', full_name='sensory.api.v1.management.GetEnrollmentsResponse.isRequestorTrusted', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=192,
  serialized_end=312,
)


_ENROLLMENTRESPONSE = _descriptor.Descriptor(
  name='EnrollmentResponse',
  full_name='sensory.api.v1.management.EnrollmentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.management.EnrollmentResponse.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='createdAt', full_name='sensory.api.v1.management.EnrollmentResponse.createdAt', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='updatedAt', full_name='sensory.api.v1.management.EnrollmentResponse.updatedAt', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='sensory.api.v1.management.EnrollmentResponse.description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='modelName', full_name='sensory.api.v1.management.EnrollmentResponse.modelName', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='modelType', full_name='sensory.api.v1.management.EnrollmentResponse.modelType', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='modelVersion', full_name='sensory.api.v1.management.EnrollmentResponse.modelVersion', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceId', full_name='sensory.api.v1.management.EnrollmentResponse.deviceId', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userId', full_name='sensory.api.v1.management.EnrollmentResponse.userId', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='compression', full_name='sensory.api.v1.management.EnrollmentResponse.compression', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='sensory.api.v1.management.EnrollmentResponse.deviceName', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='didEnrollWithLiveness', full_name='sensory.api.v1.management.EnrollmentResponse.didEnrollWithLiveness', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='referenceId', full_name='sensory.api.v1.management.EnrollmentResponse.referenceId', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='didEnrollAsBiometric', full_name='sensory.api.v1.management.EnrollmentResponse.didEnrollAsBiometric', index=13,
      number=14, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=315,
  serialized_end=756,
)


_GETENROLLMENTGROUPSRESPONSE = _descriptor.Descriptor(
  name='GetEnrollmentGroupsResponse',
  full_name='sensory.api.v1.management.GetEnrollmentGroupsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='enrollmentGroups', full_name='sensory.api.v1.management.GetEnrollmentGroupsResponse.enrollmentGroups', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=758,
  serialized_end=865,
)


_ENROLLMENTGROUPRESPONSE = _descriptor.Descriptor(
  name='EnrollmentGroupResponse',
  full_name='sensory.api.v1.management.EnrollmentGroupResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.management.EnrollmentGroupResponse.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='createdAt', full_name='sensory.api.v1.management.EnrollmentGroupResponse.createdAt', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='updatedAt', full_name='sensory.api.v1.management.EnrollmentGroupResponse.updatedAt', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sensory.api.v1.management.EnrollmentGroupResponse.name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='sensory.api.v1.management.EnrollmentGroupResponse.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='modelName', full_name='sensory.api.v1.management.EnrollmentGroupResponse.modelName', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='modelType', full_name='sensory.api.v1.management.EnrollmentGroupResponse.modelType', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='modelVersion', full_name='sensory.api.v1.management.EnrollmentGroupResponse.modelVersion', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userId', full_name='sensory.api.v1.management.EnrollmentGroupResponse.userId', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enrollments', full_name='sensory.api.v1.management.EnrollmentGroupResponse.enrollments', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=868,
  serialized_end=1209,
)


_CREATEENROLLMENTGROUPREQUEST = _descriptor.Descriptor(
  name='CreateEnrollmentGroupRequest',
  full_name='sensory.api.v1.management.CreateEnrollmentGroupRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.management.CreateEnrollmentGroupRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sensory.api.v1.management.CreateEnrollmentGroupRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\007r\005\020\001\030\377\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='sensory.api.v1.management.CreateEnrollmentGroupRequest.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\030\377\007', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='modelName', full_name='sensory.api.v1.management.CreateEnrollmentGroupRequest.modelName', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\007r\005\020\001\030\377\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userId', full_name='sensory.api.v1.management.CreateEnrollmentGroupRequest.userId', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enrollmentIds', full_name='sensory.api.v1.management.CreateEnrollmentGroupRequest.enrollmentIds', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1212,
  serialized_end=1403,
)


_APPENDENROLLMENTGROUPREQUEST = _descriptor.Descriptor(
  name='AppendEnrollmentGroupRequest',
  full_name='sensory.api.v1.management.AppendEnrollmentGroupRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='groupId', full_name='sensory.api.v1.management.AppendEnrollmentGroupRequest.groupId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enrollmentIds', full_name='sensory.api.v1.management.AppendEnrollmentGroupRequest.enrollmentIds', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1405,
  serialized_end=1486,
)


_DELETEENROLLMENTREQUEST = _descriptor.Descriptor(
  name='DeleteEnrollmentRequest',
  full_name='sensory.api.v1.management.DeleteEnrollmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.management.DeleteEnrollmentRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\260\001\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1488,
  serialized_end=1535,
)


_DELETEENROLLMENTGROUPREQUEST = _descriptor.Descriptor(
  name='DeleteEnrollmentGroupRequest',
  full_name='sensory.api.v1.management.DeleteEnrollmentGroupRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.management.DeleteEnrollmentGroupRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1537,
  serialized_end=1590,
)


_UPDATEENROLLMENTREQUEST = _descriptor.Descriptor(
  name='UpdateEnrollmentRequest',
  full_name='sensory.api.v1.management.UpdateEnrollmentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.management.UpdateEnrollmentRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\260\001\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='sensory.api.v1.management.UpdateEnrollmentRequest.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\030\377\007', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1592,
  serialized_end=1670,
)


_UPDATEENROLLMENTGROUPREQUEST = _descriptor.Descriptor(
  name='UpdateEnrollmentGroupRequest',
  full_name='sensory.api.v1.management.UpdateEnrollmentGroupRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.management.UpdateEnrollmentGroupRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sensory.api.v1.management.UpdateEnrollmentGroupRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1672,
  serialized_end=1750,
)


_REMOVEENROLLMENTSREQUEST = _descriptor.Descriptor(
  name='RemoveEnrollmentsRequest',
  full_name='sensory.api.v1.management.RemoveEnrollmentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='groupId', full_name='sensory.api.v1.management.RemoveEnrollmentsRequest.groupId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enrollmentIds', full_name='sensory.api.v1.management.RemoveEnrollmentsRequest.enrollmentIds', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005\222\001\002\010\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1752,
  serialized_end=1839,
)

_GETENROLLMENTSRESPONSE.fields_by_name['enrollments'].message_type = _ENROLLMENTRESPONSE
_ENROLLMENTRESPONSE.fields_by_name['createdAt'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ENROLLMENTRESPONSE.fields_by_name['updatedAt'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ENROLLMENTRESPONSE.fields_by_name['modelType'].enum_type = common_dot_common__pb2._MODELTYPE
_ENROLLMENTRESPONSE.fields_by_name['compression'].message_type = common_dot_common__pb2._COMPRESSIONCONFIGURATION
_GETENROLLMENTGROUPSRESPONSE.fields_by_name['enrollmentGroups'].message_type = _ENROLLMENTGROUPRESPONSE
_ENROLLMENTGROUPRESPONSE.fields_by_name['createdAt'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ENROLLMENTGROUPRESPONSE.fields_by_name['updatedAt'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ENROLLMENTGROUPRESPONSE.fields_by_name['modelType'].enum_type = common_dot_common__pb2._MODELTYPE
_ENROLLMENTGROUPRESPONSE.fields_by_name['enrollments'].message_type = _ENROLLMENTRESPONSE
DESCRIPTOR.message_types_by_name['GetEnrollmentsRequest'] = _GETENROLLMENTSREQUEST
DESCRIPTOR.message_types_by_name['GetEnrollmentsResponse'] = _GETENROLLMENTSRESPONSE
DESCRIPTOR.message_types_by_name['EnrollmentResponse'] = _ENROLLMENTRESPONSE
DESCRIPTOR.message_types_by_name['GetEnrollmentGroupsResponse'] = _GETENROLLMENTGROUPSRESPONSE
DESCRIPTOR.message_types_by_name['EnrollmentGroupResponse'] = _ENROLLMENTGROUPRESPONSE
DESCRIPTOR.message_types_by_name['CreateEnrollmentGroupRequest'] = _CREATEENROLLMENTGROUPREQUEST
DESCRIPTOR.message_types_by_name['AppendEnrollmentGroupRequest'] = _APPENDENROLLMENTGROUPREQUEST
DESCRIPTOR.message_types_by_name['DeleteEnrollmentRequest'] = _DELETEENROLLMENTREQUEST
DESCRIPTOR.message_types_by_name['DeleteEnrollmentGroupRequest'] = _DELETEENROLLMENTGROUPREQUEST
DESCRIPTOR.message_types_by_name['UpdateEnrollmentRequest'] = _UPDATEENROLLMENTREQUEST
DESCRIPTOR.message_types_by_name['UpdateEnrollmentGroupRequest'] = _UPDATEENROLLMENTGROUPREQUEST
DESCRIPTOR.message_types_by_name['RemoveEnrollmentsRequest'] = _REMOVEENROLLMENTSREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetEnrollmentsRequest = _reflection.GeneratedProtocolMessageType('GetEnrollmentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETENROLLMENTSREQUEST,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.GetEnrollmentsRequest)
  })
_sym_db.RegisterMessage(GetEnrollmentsRequest)

GetEnrollmentsResponse = _reflection.GeneratedProtocolMessageType('GetEnrollmentsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETENROLLMENTSRESPONSE,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.GetEnrollmentsResponse)
  })
_sym_db.RegisterMessage(GetEnrollmentsResponse)

EnrollmentResponse = _reflection.GeneratedProtocolMessageType('EnrollmentResponse', (_message.Message,), {
  'DESCRIPTOR' : _ENROLLMENTRESPONSE,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.EnrollmentResponse)
  })
_sym_db.RegisterMessage(EnrollmentResponse)

GetEnrollmentGroupsResponse = _reflection.GeneratedProtocolMessageType('GetEnrollmentGroupsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETENROLLMENTGROUPSRESPONSE,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.GetEnrollmentGroupsResponse)
  })
_sym_db.RegisterMessage(GetEnrollmentGroupsResponse)

EnrollmentGroupResponse = _reflection.GeneratedProtocolMessageType('EnrollmentGroupResponse', (_message.Message,), {
  'DESCRIPTOR' : _ENROLLMENTGROUPRESPONSE,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.EnrollmentGroupResponse)
  })
_sym_db.RegisterMessage(EnrollmentGroupResponse)

CreateEnrollmentGroupRequest = _reflection.GeneratedProtocolMessageType('CreateEnrollmentGroupRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEENROLLMENTGROUPREQUEST,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.CreateEnrollmentGroupRequest)
  })
_sym_db.RegisterMessage(CreateEnrollmentGroupRequest)

AppendEnrollmentGroupRequest = _reflection.GeneratedProtocolMessageType('AppendEnrollmentGroupRequest', (_message.Message,), {
  'DESCRIPTOR' : _APPENDENROLLMENTGROUPREQUEST,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.AppendEnrollmentGroupRequest)
  })
_sym_db.RegisterMessage(AppendEnrollmentGroupRequest)

DeleteEnrollmentRequest = _reflection.GeneratedProtocolMessageType('DeleteEnrollmentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEENROLLMENTREQUEST,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.DeleteEnrollmentRequest)
  })
_sym_db.RegisterMessage(DeleteEnrollmentRequest)

DeleteEnrollmentGroupRequest = _reflection.GeneratedProtocolMessageType('DeleteEnrollmentGroupRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEENROLLMENTGROUPREQUEST,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.DeleteEnrollmentGroupRequest)
  })
_sym_db.RegisterMessage(DeleteEnrollmentGroupRequest)

UpdateEnrollmentRequest = _reflection.GeneratedProtocolMessageType('UpdateEnrollmentRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEENROLLMENTREQUEST,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.UpdateEnrollmentRequest)
  })
_sym_db.RegisterMessage(UpdateEnrollmentRequest)

UpdateEnrollmentGroupRequest = _reflection.GeneratedProtocolMessageType('UpdateEnrollmentGroupRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEENROLLMENTGROUPREQUEST,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.UpdateEnrollmentGroupRequest)
  })
_sym_db.RegisterMessage(UpdateEnrollmentGroupRequest)

RemoveEnrollmentsRequest = _reflection.GeneratedProtocolMessageType('RemoveEnrollmentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEENROLLMENTSREQUEST,
  '__module__' : 'v1.management.enrollment_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.RemoveEnrollmentsRequest)
  })
_sym_db.RegisterMessage(RemoveEnrollmentsRequest)


DESCRIPTOR._options = None
_GETENROLLMENTSREQUEST.fields_by_name['userId']._options = None
_CREATEENROLLMENTGROUPREQUEST.fields_by_name['id']._options = None
_CREATEENROLLMENTGROUPREQUEST.fields_by_name['name']._options = None
_CREATEENROLLMENTGROUPREQUEST.fields_by_name['description']._options = None
_CREATEENROLLMENTGROUPREQUEST.fields_by_name['modelName']._options = None
_CREATEENROLLMENTGROUPREQUEST.fields_by_name['userId']._options = None
_APPENDENROLLMENTGROUPREQUEST.fields_by_name['groupId']._options = None
_DELETEENROLLMENTREQUEST.fields_by_name['id']._options = None
_DELETEENROLLMENTGROUPREQUEST.fields_by_name['id']._options = None
_UPDATEENROLLMENTREQUEST.fields_by_name['id']._options = None
_UPDATEENROLLMENTREQUEST.fields_by_name['description']._options = None
_UPDATEENROLLMENTGROUPREQUEST.fields_by_name['id']._options = None
_UPDATEENROLLMENTGROUPREQUEST.fields_by_name['name']._options = None
_REMOVEENROLLMENTSREQUEST.fields_by_name['groupId']._options = None
_REMOVEENROLLMENTSREQUEST.fields_by_name['enrollmentIds']._options = None

_ENROLLMENTSERVICE = _descriptor.ServiceDescriptor(
  name='EnrollmentService',
  full_name='sensory.api.v1.management.EnrollmentService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1842,
  serialized_end=3042,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetEnrollments',
    full_name='sensory.api.v1.management.EnrollmentService.GetEnrollments',
    index=0,
    containing_service=None,
    input_type=_GETENROLLMENTSREQUEST,
    output_type=_GETENROLLMENTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEnrollmentGroups',
    full_name='sensory.api.v1.management.EnrollmentService.GetEnrollmentGroups',
    index=1,
    containing_service=None,
    input_type=_GETENROLLMENTSREQUEST,
    output_type=_GETENROLLMENTGROUPSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateEnrollmentGroup',
    full_name='sensory.api.v1.management.EnrollmentService.CreateEnrollmentGroup',
    index=2,
    containing_service=None,
    input_type=_CREATEENROLLMENTGROUPREQUEST,
    output_type=_ENROLLMENTGROUPRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AppendEnrollmentGroup',
    full_name='sensory.api.v1.management.EnrollmentService.AppendEnrollmentGroup',
    index=3,
    containing_service=None,
    input_type=_APPENDENROLLMENTGROUPREQUEST,
    output_type=_ENROLLMENTGROUPRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteEnrollment',
    full_name='sensory.api.v1.management.EnrollmentService.DeleteEnrollment',
    index=4,
    containing_service=None,
    input_type=_DELETEENROLLMENTREQUEST,
    output_type=_ENROLLMENTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteEnrollmentGroup',
    full_name='sensory.api.v1.management.EnrollmentService.DeleteEnrollmentGroup',
    index=5,
    containing_service=None,
    input_type=_DELETEENROLLMENTGROUPREQUEST,
    output_type=_ENROLLMENTGROUPRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateEnrollment',
    full_name='sensory.api.v1.management.EnrollmentService.UpdateEnrollment',
    index=6,
    containing_service=None,
    input_type=_UPDATEENROLLMENTREQUEST,
    output_type=_ENROLLMENTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateEnrollmentGroup',
    full_name='sensory.api.v1.management.EnrollmentService.UpdateEnrollmentGroup',
    index=7,
    containing_service=None,
    input_type=_UPDATEENROLLMENTGROUPREQUEST,
    output_type=_ENROLLMENTGROUPRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveEnrollmentsFromGroup',
    full_name='sensory.api.v1.management.EnrollmentService.RemoveEnrollmentsFromGroup',
    index=8,
    containing_service=None,
    input_type=_REMOVEENROLLMENTSREQUEST,
    output_type=_ENROLLMENTGROUPRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_ENROLLMENTSERVICE)

DESCRIPTOR.services_by_name['EnrollmentService'] = _ENROLLMENTSERVICE

# @@protoc_insertion_point(module_scope)



import os
import json
import binascii
import typing
import base64
import configparser
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey

from sensory_cloud.config import (
    Config,
    CloudHost,
    EnrollmentType,
    SDKConfig,
)
from sensory_cloud.services.oauth_service import (
    OauthClient,
    OauthService,
    ISecureCredentialStore,
    GenericCredentialStore,
)

import sensory_cloud.generated.v1.management.device_pb2 as device_pb2


class Initializer:
    """
    Initialization class used to register a device
    """

    _enrollment_type_map: dict = {
        "none": EnrollmentType.none,
        "sharedSecret": EnrollmentType.shared_secret,
        "jwt": EnrollmentType.jwt,
    }

    def __init__(
        self,
        init_config: typing.Union[str, SDKConfig],
        secure_credential_store: ISecureCredentialStore = None,
    ):
        """
        Constructor method to generate an Initializer object

        Arguments:
            init_config (str or SDKConfig): The init_config argument can either be a string
                containing the path to a .ini config file or an SDKConfig object containing
                the required configuration information
            secure_credential_store (ISecureCredentialStore): The secure_credential_store
                argument is an ISecureCredentialStore abstract class object.  If it is set to
                None then the client credentials will be written to a GenericCredentialStore
                object
        """

        self.init_config: typing.Union[str, SDKConfig] = init_config
        self.secure_credential_store: ISecureCredentialStore = secure_credential_store
        self.sdk_config: SDKConfig = None
        self.config_parser: configparser.ConfigParser = None

    def _read_config_from_file(self) -> None:
        """
        Private method that sets the sdk_config attribute in the case that the init_config
        attribute is a path to a config file
        """

        self.config_parser: configparser.ConfigParser = configparser.ConfigParser()
        self.config_parser.read(self.init_config)

        sdk_config_parser: configparser.SectionProxy = self.config_parser[
            "SDK-configuration"
        ]

        self.sdk_config: SDKConfig = SDKConfig(
            fully_qualified_domain_name=sdk_config_parser.get(
                "fullyQualifiedDomainName"
            ),
            tenant_id=sdk_config_parser.get("tenantId"),
            is_connection_secure=sdk_config_parser.getboolean("isSecure"),
            enrollment_type=self._enrollment_type_map[
                sdk_config_parser.get("enrollmentType")
            ],
            credential=sdk_config_parser.get("credential"),
            device_id=sdk_config_parser.get("deviceId"),
            device_name=sdk_config_parser.get("deviceName"),
        )

        if (
            "client-configuration" in self.config_parser
            and self.secure_credential_store is None
        ):
            client_config_parser: configparser.SectionProxy = self.config_parser[
                "client-configuration"
            ]
            self.secure_credential_store: GenericCredentialStore = (
                GenericCredentialStore(
                    client_id=client_config_parser.get("clientId"),
                    client_secret=client_config_parser.get("clientSecret"),
                )
            )

    def _get_config(self) -> Config:
        """
        Private method that creates a Config object from the sdk_config attribute

        Returns:
            Config object used to construct an OauthService object
        """

        cloud_host: CloudHost = CloudHost(
            host=self.sdk_config.fully_qualified_domain_name,
            is_connection_secure=self.sdk_config.is_connection_secure,
        )

        config: Config = Config(
            tenant_id=self.sdk_config.tenant_id, cloud_host=cloud_host
        )
        config.connect()

        return config

    def _get_credential(self) -> str:
        """
        Private method that generates a credential string depending on the
        enrollment type

        Returns:
            Credential string used to enroll the device
        """

        if self.sdk_config.enrollment_type == EnrollmentType.none:
            credential: str = ""
        elif self.sdk_config.enrollment_type == EnrollmentType.shared_secret:
            credential: str = self.sdk_config.credential
        elif self.sdk_config.enrollment_type == EnrollmentType.jwt:
            credential: str = self._generate_jwt(private_key=self.sdk_config.credential)

        return credential

    def _generate_jwt(self, private_key: str) -> str:
        """
        Private method that generates a jwt token in the case that the enrollment
        type is 'jwt'

        Returns:
            A jwt token used as the enrollment credential
        """

        header: dict = {
            "alg": "EdDSA",
            "typ": "JWT",
        }
        payload: dict = {
            "name": self.sdk_config.device_name,
            "tenant": self.sdk_config.tenant_id,
            "client": self.secure_credential_store.client_id,
        }

        signing_key: Ed25519PrivateKey = Ed25519PrivateKey.from_private_bytes(
            binascii.unhexlify(private_key.encode("utf-8"))
        )

        header_base64: bytes = base64.urlsafe_b64encode(
            json.dumps(header).encode("utf-8")
        )
        payload_base64: bytes = base64.urlsafe_b64encode(
            json.dumps(payload).encode("utf-8")
        )
        to_sign: bytes = header_base64 + b"." + payload_base64

        signature: bytes = base64.urlsafe_b64encode(signing_key.sign(to_sign))

        token: str = (
            b".".join([header_base64, payload_base64, signature])
            .decode()
            .replace("+", "-")
            .replace("/", "_")
            .replace("=", "")
        )

        return token

    def _update_config_file(self) -> None:
        """
        Private method that updates the config file with the client id and the
        client secret upon successful enrollment of the device
        """

        if "client-configuration" not in self.config_parser:
            self.config_parser.add_section("client-configuration")

        self.config_parser.set(
            "client-configuration", "clientId", self.secure_credential_store.client_id
        )
        self.config_parser.set(
            "client-configuration",
            "clientSecret",
            self.secure_credential_store.client_secret,
        )

        with open(self.init_config, "w") as config_file:
            self.config_parser.write(config_file)

    def initialize(self) -> typing.Union[device_pb2.DeviceResponse, Exception]:
        """
        Public method that uses the sdk and client credentials to enroll a
        new device

        Returns:
            If the device enrollment is successful then a device_pb2.DeviceResponse
            object will be returned.  If the device is already enrolled or another
            device is enrolled with the client credentials specified, then an
            Exception will be returned.
        """

        if isinstance(self.init_config, str):
            if not os.path.exists(self.init_config):
                raise Exception(f"The path, {self.init_config}, does not exist")
            self._read_config_from_file()
        elif isinstance(self.init_config, SDKConfig):
            self.sdk_config = self.init_config
        else:
            raise Exception(
                "The init_config attribute must be a string containing the config file path or an SDKConfig object"
            )

        if self.secure_credential_store is None:
            client_credentials: OauthClient = OauthService.generate_credentials()
            self.secure_credential_store: GenericCredentialStore = (
                GenericCredentialStore(
                    client_id=client_credentials.client_id,
                    client_secret=client_credentials.client_secret,
                )
            )

        config: Config = self._get_config()
        oauth_service: OauthService = OauthService(
            config=config, secure_credential_store=self.secure_credential_store
        )

        try:
            device_response = oauth_service.get_who_am_i()
            device_is_registered = self.sdk_config.device_id == device_response.deviceId
            if not device_is_registered:
                err: str = f"Another device with deviceId = {device_response.deviceId} is already enrolled with this client"
                print(err)
                return Exception(err)
        except Exception as e:
            device_is_registered = False

        if not device_is_registered:
            credential: str = self._get_credential()
            response: device_pb2.DeviceResponse = oauth_service.register(
                device_id=self.sdk_config.device_id,
                device_name=self.sdk_config.device_name,
                credential=credential,
            )
            if self.config_parser is not None:
                self._update_config_file()
        else:
            err: str = "This device is already enrolled"
            print(err)
            response: Exception = Exception(err)

        return response



import unittest
from unittest.mock import MagicMock

from sensory_cloud.config import Config, CloudHost
from sensory_cloud.generated.v1.video.video_pb2 import (
    CreateEnrollmentConfig,
    CreateEnrollmentRequest,
    CreateEnrollmentResponse,
    GetModelsResponse,
    LivenessRecognitionResponse,
    RecognitionThreshold,
    ValidateRecognitionConfig,
    ValidateRecognitionRequest,
    VideoModel,
    AuthenticateConfig,
    AuthenticateRequest,
    AuthenticateResponse,
)
from sensory_cloud.generated.v1.video.video_pb2_grpc import (
    VideoBiometricsStub,
    VideoModelsStub,
    VideoRecognitionStub,
)
from sensory_cloud.token_manager import ITokenManager, TokenManager
from sensory_cloud.services.oauth_service import ISecureCredentialStore, OauthService
from sensory_cloud.services.video_service import VideoService


class MockCredentialStore(ISecureCredentialStore):
    def __init__(self, client_id, client_secret):
        self._client_id = client_id
        self._client_secret = client_secret

    @property
    def client_id(self):
        return self._client_id

    @property
    def client_secret(self):
        return self._client_secret


class MockVidoService(VideoService):
    def __init__(
        self,
        config: Config,
        token_manager: ITokenManager,
        video_models_client: VideoModelsStub,
        video_biometrics_client: VideoBiometricsStub,
        video_recognition_client: VideoRecognitionStub,
    ):
        self.config: Config = config
        self._token_manager: ITokenManager = token_manager
        self._video_models_client: VideoModelsStub = video_models_client
        self._video_biometrics_client: VideoBiometricsStub = video_biometrics_client
        self._video_recognition_client: VideoRecognitionStub = video_recognition_client


class VideoServiceTest(unittest.TestCase):
    cloud_host: CloudHost = CloudHost(host="domain.name")
    config: Config = Config(cloud_host=cloud_host, tenant_id="tenant-id")
    config.connect()

    credential_store: MockCredentialStore = MockCredentialStore(
        client_id="client-id", client_secret="client-secret"
    )
    oauth_service: OauthService = OauthService(
        config=config, secure_credential_store=credential_store
    )
    token_manager: TokenManager = TokenManager(oauth_service=oauth_service)
    token_manager.get_authorization_metadata = MagicMock(return_value=None)

    video_models_client: VideoModelsStub = VideoModelsStub(config.channel)
    video_biometrics_client: VideoBiometricsStub = VideoBiometricsStub(config.channel)
    video_recognition_client: VideoRecognitionStub = VideoRecognitionStub(
        config.channel
    )

    def test_get_models(self):
        self.config.connect()

        response: GetModelsResponse = GetModelsResponse(
            models=[VideoModel(name="model-name")]
        )

        self.video_models_client.GetModels = MagicMock(return_value=response)

        video_service: MockVidoService = MockVidoService(
            config=self.config,
            token_manager=self.token_manager,
            video_models_client=self.video_models_client,
            video_biometrics_client=self.video_biometrics_client,
            video_recognition_client=self.video_recognition_client,
        )

        models_response: GetModelsResponse = video_service.get_models()

        self.assertEqual(models_response, response)

        self.config.channel.close()

    def test_stream_enrollment(self):
        self.config.connect()

        description: str = "my-description"
        user_id: str = "user-id"
        model_name: str = "my-model"
        device_id: str = "my-device"
        is_liveness_enabled: bool = False
        threshold: RecognitionThreshold = RecognitionThreshold.Value("HIGH")

        enrollment_config: CreateEnrollmentConfig = CreateEnrollmentConfig(
            description=description,
            userId=user_id,
            deviceId=device_id,
            modelName=model_name,
            isLivenessEnabled=is_liveness_enabled,
            livenessThreshold=threshold,
        )

        mock_request: CreateEnrollmentRequest = CreateEnrollmentRequest(
            config=enrollment_config
        )
        mock_response: CreateEnrollmentResponse = CreateEnrollmentResponse()

        self.video_biometrics_client.CreateEnrollment = MagicMock(
            return_value=(mock_request, mock_response)
        )

        video_service: MockVidoService = MockVidoService(
            config=self.config,
            token_manager=self.token_manager,
            video_models_client=self.video_models_client,
            video_biometrics_client=self.video_biometrics_client,
            video_recognition_client=self.video_recognition_client,
        )

        (
            enrollment_stream_request,
            enrollment_stream_response,
        ) = video_service.stream_enrollment(
            description=description,
            user_id=user_id,
            model_name=model_name,
            device_id=device_id,
            is_liveness_enabled=is_liveness_enabled,
            video_stream_iterator=None,
            threshold=threshold,
        )

        self.assertIsNotNone(enrollment_stream_response)

        config_message = enrollment_stream_request.config

        self.assertEqual(config_message.description, description)
        self.assertEqual(config_message.userId, user_id)
        self.assertEqual(config_message.modelName, model_name)
        self.assertEqual(config_message.isLivenessEnabled, is_liveness_enabled)

        self.config.channel.close()

    def test_stream_authentication(self):
        self.config.connect()

        enrollment_id: str = "enrollment-id"
        is_liveness_enabled: bool = False
        threshold: RecognitionThreshold = RecognitionThreshold.Value("HIGH")

        authenticate_config: AuthenticateConfig = AuthenticateConfig(
            enrollmentId=enrollment_id,
            isLivenessEnabled=is_liveness_enabled,
            livenessThreshold=threshold,
        )

        mock_request: AuthenticateRequest = AuthenticateRequest(
            config=authenticate_config
        )
        mock_response: AuthenticateResponse = AuthenticateResponse()

        self.video_biometrics_client.Authenticate = MagicMock(
            return_value=(mock_request, mock_response)
        )

        video_service: MockVidoService = MockVidoService(
            config=self.config,
            token_manager=self.token_manager,
            video_models_client=self.video_models_client,
            video_biometrics_client=self.video_biometrics_client,
            video_recognition_client=self.video_recognition_client,
        )

        (
            authenticate_stream_request,
            authenticate_stream_response,
        ) = video_service.stream_authentication(
            enrollment_id=enrollment_id,
            is_liveness_enabled=is_liveness_enabled,
            video_stream_iterator=None,
            threshold=threshold,
        )

        self.assertIsNotNone(authenticate_stream_response)

        config_message = authenticate_stream_request.config

        self.assertEqual(config_message.enrollmentId, enrollment_id)
        self.assertEqual(config_message.livenessThreshold, threshold)
        self.assertEqual(config_message.isLivenessEnabled, is_liveness_enabled)

        self.config.channel.close()

    def test_stream_group_authentication(self):
        self.config.connect()

        enrollment_group_id: str = "enrollment-group-id"
        is_liveness_enabled: bool = False
        threshold: RecognitionThreshold = RecognitionThreshold.Value("HIGH")

        authenticate_config: AuthenticateConfig = AuthenticateConfig(
            enrollmentGroupId=enrollment_group_id,
            isLivenessEnabled=is_liveness_enabled,
            livenessThreshold=threshold,
        )

        mock_request: AuthenticateRequest = AuthenticateRequest(
            config=authenticate_config
        )
        mock_response: AuthenticateResponse = AuthenticateResponse()

        self.video_biometrics_client.Authenticate = MagicMock(
            return_value=(mock_request, mock_response)
        )

        video_service: MockVidoService = MockVidoService(
            config=self.config,
            token_manager=self.token_manager,
            video_models_client=self.video_models_client,
            video_biometrics_client=self.video_biometrics_client,
            video_recognition_client=self.video_recognition_client,
        )

        (
            authenticate_stream_request,
            authenticate_stream_response,
        ) = video_service.stream_group_authentication(
            enrollment_group_id=enrollment_group_id,
            is_liveness_enabled=is_liveness_enabled,
            video_stream_iterator=None,
            threshold=threshold,
        )

        self.assertIsNotNone(authenticate_stream_response)

        config_message = authenticate_stream_request.config

        self.assertEqual(config_message.enrollmentGroupId, enrollment_group_id)
        self.assertEqual(config_message.livenessThreshold, threshold)
        self.assertEqual(config_message.isLivenessEnabled, is_liveness_enabled)

        self.config.channel.close()

    def test_stream_recognition(self):
        self.config.connect()

        user_id: str = "user-id"
        model_name: str = "my-model"
        threshold: RecognitionThreshold = RecognitionThreshold.Value("LOW")

        recognition_config: ValidateRecognitionConfig = ValidateRecognitionConfig(
            userId=user_id, modelName=model_name, threshold=threshold
        )

        mock_request: ValidateRecognitionRequest = ValidateRecognitionRequest(
            config=recognition_config
        )
        mock_response: LivenessRecognitionResponse = LivenessRecognitionResponse()

        self.video_recognition_client.ValidateLiveness = MagicMock(
            return_value=(mock_request, mock_response)
        )

        video_service: MockVidoService = MockVidoService(
            config=self.config,
            token_manager=self.token_manager,
            video_models_client=self.video_models_client,
            video_biometrics_client=self.video_biometrics_client,
            video_recognition_client=self.video_recognition_client,
        )

        (
            liveness_recognition_request,
            liveness_recognition_response,
        ) = video_service.stream_liveness_recognition(
            user_id=user_id,
            model_name=model_name,
            video_stream_iterator=None,
            threshold=threshold,
        )

        self.assertIsNotNone(liveness_recognition_response)

        config_message = liveness_recognition_request.config

        self.assertEqual(config_message.userId, user_id)
        self.assertEqual(config_message.threshold, threshold)
        self.assertEqual(config_message.modelName, model_name)

        self.config.channel.close()



import secrets


class CryptoService:
    """
    Class that handles cryptographic operations
    """

    _allowable_characters: str = (
        "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
    )

    def get_secure_random_string(self, length: int) -> str:
        """
        Method that generates a cryptographically-random string

        Arguments:
            length: Integer denoting the length of the string that is returned

        Returns:
            A cryptographically-random string
        """

        return "".join(
            secrets.choice(self._allowable_characters) for i in range(length)
        )
